"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var QueryBuilder_1 = require("../query-builder/QueryBuilder");
var MysqlSchemaBuilder_1 = require("../schema-builder/MysqlSchemaBuilder");
var ConnectionIsNotSetError_1 = require("./error/ConnectionIsNotSetError");
var BaseDriver_1 = require("./BaseDriver");
var ColumnTypes_1 = require("../metadata-builder/types/ColumnTypes");
var moment = require("moment");
/**
 * This driver organizes work with mysql database.
 */
var MysqlDriver = (function (_super) {
    __extends(MysqlDriver, _super);
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function MysqlDriver(mysql) {
        _super.call(this);
        // if driver dependency is not given explicitly, then try to load it via "require"
        if (!mysql && require) {
            try {
                mysql = require("mysql");
            }
            catch (e) {
                throw new Error("Mysql package was not found installed. Try to install it: npm install mysql --save");
            }
        }
        else {
            throw new Error("Cannot load mysql driver dependencies. Try to install all required dependencies.");
        }
        this.mysql = mysql;
    }
    Object.defineProperty(MysqlDriver.prototype, "native", {
        // -------------------------------------------------------------------------
        // Getter Methods
        // -------------------------------------------------------------------------
        /**
         * Access to the native implementation of the database.
         */
        get: function () {
            return this.mysql;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MysqlDriver.prototype, "nativeConnection", {
        /**
         * Access to the native implementation of the database.
         */
        get: function () {
            return this.mysqlConnection;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MysqlDriver.prototype, "db", {
        /**
         * Database name to which this connection is made.
         */
        get: function () {
            return this.connection.options.database;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a query builder which can be used to build an sql queries.
     */
    MysqlDriver.prototype.createQueryBuilder = function () {
        return new QueryBuilder_1.QueryBuilder(this.connection);
    };
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    MysqlDriver.prototype.createSchemaBuilder = function () {
        return new MysqlSchemaBuilder_1.MysqlSchemaBuilder(this);
    };
    /**
     * Performs connection to the database based on given connection options.
     */
    MysqlDriver.prototype.connect = function () {
        var _this = this;
        this.mysqlConnection = this.mysql.createConnection({
            host: this.connection.options.host,
            user: this.connection.options.username,
            password: this.connection.options.password,
            database: this.connection.options.database
        });
        return new Promise(function (ok, fail) {
            _this.mysqlConnection.connect(function (err) { return err ? fail(err) : ok(); });
        });
    };
    /**
     * Closes connection with database.
     */
    MysqlDriver.prototype.disconnect = function () {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        return new Promise(function (ok, fail) {
            _this.mysqlConnection.end(function (err) { return err ? fail(err) : ok(); });
        });
    };
    /**
     * Executes a given SQL query.
     */
    MysqlDriver.prototype.query = function (query) {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        this.logQuery(query);
        return new Promise(function (ok, fail) { return _this.mysqlConnection.query(query, function (err, result) {
            if (err) {
                _this.logFailedQuery(query);
                _this.logQueryError(err);
                fail(err);
            }
            else {
                ok(result);
            }
        }); });
    };
    /**
     * Clears all tables in the currently connected database.
     */
    MysqlDriver.prototype.clearDatabase = function () {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var disableForeignKeysCheckQuery = "SET FOREIGN_KEY_CHECKS = 0;";
        var dropTablesQuery = "SELECT concat('DROP TABLE IF EXISTS ', table_name, ';') AS q FROM " +
            ("information_schema.tables WHERE table_schema = '" + this.db + "';");
        var enableForeignKeysCheckQuery = "SET FOREIGN_KEY_CHECKS = 1;";
        return this.query(disableForeignKeysCheckQuery)
            .then(function () { return _this.query(dropTablesQuery); })
            .then(function (results) { return Promise.all(results.map(function (q) { return _this.query(q["q"]); })); })
            .then(function () { return _this.query(enableForeignKeysCheckQuery); })
            .then(function () { });
    };
    /**
     * Updates rows that match given conditions in the given table.
     */
    MysqlDriver.prototype.update = function (tableName, valuesMap, conditions) {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var qb = this.createQueryBuilder().update(tableName, valuesMap).from(tableName, "t");
        Object.keys(conditions).forEach(function (key) { return qb.andWhere(key + "=:" + key, (_a = {}, _a[key] = conditions[key], _a)); var _a; });
        return qb.execute().then(function () { });
    };
    /**
     * Insert a new row into given table.
     */
    MysqlDriver.prototype.insert = function (tableName, keyValues) {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var columns = Object.keys(keyValues).join(",");
        var values = Object.keys(keyValues).map(function (key) { return keyValues[key]; }).join(",");
        var query = "INSERT INTO " + tableName + "(" + columns + ") VALUES (" + values + ")";
        return this.query(query);
    };
    /**
     * Deletes from the given table by a given conditions.
     */
    MysqlDriver.prototype.delete = function (tableName, conditions) {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var qb = this.createQueryBuilder().delete(tableName);
        Object.keys(conditions).forEach(function (key) { return qb.andWhere(key + "=:" + key, (_a = {}, _a[key] = conditions[key], _a)); var _a; });
        return qb.execute().then(function () { });
    };
    /**
     * Starts mysql transaction.
     */
    MysqlDriver.prototype.beginTransaction = function () {
        return this.query("START TRANSACTION").then(function () { });
    };
    /**
     * Ends mysql transaction.
     */
    MysqlDriver.prototype.endTransaction = function () {
        return this.query("COMMIT").then(function () { });
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    MysqlDriver.prototype.preparePersistentValue = function (value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.BOOLEAN:
                return value === true ? 1 : 0;
            case ColumnTypes_1.ColumnTypes.DATE:
                return moment(value).format("YYYY-MM-DD");
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value).format("hh:mm:ss");
            case ColumnTypes_1.ColumnTypes.DATETIME:
                return moment(value).format("YYYY-MM-DD hh:mm:ss");
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.stringify(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value
                    .map(function (i) { return String(i); })
                    .join(",");
        }
        return value;
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    MysqlDriver.prototype.prepareHydratedValue = function (value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.DATE:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD").toDate();
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value, "hh:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.DATETIME:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD hh:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.parse(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value.split(",");
        }
        return value;
    };
    /**
     * Escapes given value.
     */
    MysqlDriver.prototype.escape = function (value) {
        return this.mysqlConnection.escape(value);
    };
    return MysqlDriver;
}(BaseDriver_1.BaseDriver));
exports.MysqlDriver = MysqlDriver;
//# sourceMappingURL=MysqlDriver.js.map