"use strict";
var MetadataAlreadyExistsError_1 = require("./error/MetadataAlreadyExistsError");
var MetadataWithSuchNameAlreadyExistsError_1 = require("./error/MetadataWithSuchNameAlreadyExistsError");
/**
 * Storage all metadatas of all available types: tables, fields, subscribers, relations, etc.
 * Each metadata represents some specifications of what it represents.
 */
var MetadataStorage = (function () {
    function MetadataStorage() {
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this._tableMetadatas = [];
        this._eventSubscriberMetadatas = [];
        this._columnMetadatas = [];
        this._indexMetadatas = [];
        this._entityListenerMetadatas = [];
        this._compoundIndexMetadatas = [];
        this._relationMetadatas = [];
    }
    Object.defineProperty(MetadataStorage.prototype, "tableMetadatas", {
        // -------------------------------------------------------------------------
        // Getter Methods
        // -------------------------------------------------------------------------
        get: function () {
            return this._tableMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "eventSubscriberMetadatas", {
        get: function () {
            return this._eventSubscriberMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "columnMetadatas", {
        get: function () {
            return this._columnMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "indexMetadatas", {
        get: function () {
            return this._indexMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "entityListenerMetadatas", {
        get: function () {
            return this._entityListenerMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "compoundIndexMetadatas", {
        get: function () {
            return this._compoundIndexMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MetadataStorage.prototype, "relationMetadatas", {
        get: function () {
            return this._relationMetadatas;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Adder Methods
    // -------------------------------------------------------------------------
    MetadataStorage.prototype.addTableMetadata = function (metadata) {
        if (this.hasTableMetadataWithObjectConstructor(metadata.target))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("Table", metadata.target);
        if (metadata.name && this.hasTableMetadataWithName(metadata.name))
            throw new MetadataWithSuchNameAlreadyExistsError_1.MetadataWithSuchNameAlreadyExistsError("Table", metadata.name);
        this.tableMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addRelationMetadata = function (metadata) {
        if (this.hasRelationWithOneMetadataOnProperty(metadata.target, metadata.propertyName))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("RelationMetadata", metadata.target, metadata.propertyName);
        if (metadata.name && this.hasRelationWithOneMetadataWithName(metadata.target, metadata.name))
            throw new MetadataWithSuchNameAlreadyExistsError_1.MetadataWithSuchNameAlreadyExistsError("RelationMetadata", metadata.name);
        this.relationMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addColumnMetadata = function (metadata) {
        if (this.hasFieldMetadataOnProperty(metadata.target, metadata.propertyName))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("Column", metadata.target);
        if (metadata.name && this.hasFieldMetadataWithName(metadata.target, metadata.name))
            throw new MetadataWithSuchNameAlreadyExistsError_1.MetadataWithSuchNameAlreadyExistsError("Column", metadata.name);
        this.columnMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addEventSubscriberMetadata = function (metadata) {
        if (this.hasEventSubscriberWithObjectConstructor(metadata.target))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("EventSubscriber", metadata.target);
        this.eventSubscriberMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addIndexMetadata = function (metadata) {
        if (this.hasFieldMetadataOnProperty(metadata.target, metadata.propertyName))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("Index", metadata.target);
        if (metadata.name && this.hasFieldMetadataWithName(metadata.target, metadata.name))
            throw new MetadataWithSuchNameAlreadyExistsError_1.MetadataWithSuchNameAlreadyExistsError("Index", metadata.name);
        this.indexMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addCompoundIndexMetadata = function (metadata) {
        if (this.hasCompoundIndexMetadataWithObjectConstructor(metadata.target))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("CompoundIndex", metadata.target);
        this.compoundIndexMetadatas.push(metadata);
    };
    MetadataStorage.prototype.addEntityListenerMetadata = function (metadata) {
        if (this.hasFieldMetadataOnProperty(metadata.target, metadata.propertyName))
            throw new MetadataAlreadyExistsError_1.MetadataAlreadyExistsError("EventListener", metadata.target);
        this.entityListenerMetadatas.push(metadata);
    };
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    MetadataStorage.prototype.findEventSubscribersForClasses = function (classes) {
        return this.eventSubscriberMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findEntityListenersForClasses = function (classes) {
        return this.entityListenerMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findTableMetadatasForClasses = function (classes) {
        return this.tableMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1 && !metadata.isAbstract; });
    };
    MetadataStorage.prototype.findCompoundIndexMetadatasForClasses = function (classes) {
        return this.compoundIndexMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findAbstractTableMetadatasForClasses = function (classes) {
        return this.tableMetadatas.filter(function (metadata) { return metadata.isAbstract && classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findIndexMetadatasForClasses = function (classes) {
        return this.indexMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findFieldMetadatasForClasses = function (classes) {
        return this.columnMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    MetadataStorage.prototype.findRelationMetadatasForClasses = function (classes) {
        return this.relationMetadatas.filter(function (metadata) { return classes.indexOf(metadata.target) !== -1; });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    MetadataStorage.prototype.hasTableMetadataWithObjectConstructor = function (constructor) {
        return !!this.tableMetadatas.find(function (metadata) { return metadata.target === constructor; });
    };
    MetadataStorage.prototype.hasCompoundIndexMetadataWithObjectConstructor = function (constructor) {
        return !!this.compoundIndexMetadatas.find(function (metadata) { return metadata.target === constructor; });
    };
    MetadataStorage.prototype.hasEventSubscriberWithObjectConstructor = function (constructor) {
        return !!this.eventSubscriberMetadatas.find(function (metadata) { return metadata.target === constructor; });
    };
    MetadataStorage.prototype.hasFieldMetadataOnProperty = function (constructor, propertyName) {
        return !!this.columnMetadatas.find(function (metadata) { return metadata.target === constructor && metadata.propertyName === propertyName; });
    };
    MetadataStorage.prototype.hasRelationWithOneMetadataOnProperty = function (constructor, propertyName) {
        return !!this.relationMetadatas.find(function (metadata) { return metadata.target === constructor && metadata.propertyName === propertyName; });
    };
    MetadataStorage.prototype.hasTableMetadataWithName = function (name) {
        return !!this.tableMetadatas.find(function (metadata) { return metadata.name === name; });
    };
    MetadataStorage.prototype.hasFieldMetadataWithName = function (constructor, name) {
        return !!this.columnMetadatas.find(function (metadata) { return metadata.target === constructor && metadata.name === name; });
    };
    MetadataStorage.prototype.hasRelationWithOneMetadataWithName = function (constructor, name) {
        return !!this.relationMetadatas.find(function (metadata) { return metadata.target === constructor && metadata.name === name; });
    };
    return MetadataStorage;
}());
exports.MetadataStorage = MetadataStorage;
/**
 * Default metadata storage used as singleton and can be used to storage all metadatas in the system.
 */
exports.defaultMetadataStorage = new MetadataStorage();
//# sourceMappingURL=MetadataStorage.js.map