import { PropertyMetadata } from "./PropertyMetadata";
import { ColumnOptions } from "../options/ColumnOptions";
import { NamingStrategy } from "../../naming-strategy/NamingStrategy";
import { ColumnType } from "../types/ColumnTypes";
/**
 * Constructor arguments for ColumnMetadata class.
 */
export interface ColumnMetadataArgs {
    /**
     * Class to which this column is applied.
     */
    target: Function;
    /**
     * Class's property name to which this column is applied.
     */
    propertyName: string;
    /**
     * Class's property type (reflected) to which this column is applied.
     */
    propertyType: string;
    /**
     * Indicates if this column is primary key or not.
     */
    isPrimaryKey?: boolean;
    /**
     * Indicates if this column is create date column or not.
     */
    isCreateDate?: boolean;
    /**
     * Indicates if this column is update date column or not.
     */
    isUpdateDate?: boolean;
    /**
     * Indicates if this column is virtual or not.
     */
    isVirtual?: boolean;
    /**
     * Extra column options.
     */
    options: ColumnOptions;
}
/**
 * This metadata contains all information about class's column.
 */
export declare class ColumnMetadata extends PropertyMetadata {
    /**
     * Naming strategy to be used to generate column name.
     */
    namingStrategy: NamingStrategy;
    /**
     * Column name to be used in the database.
     */
    private _name;
    /**
     * The real reflected property type.
     */
    private _propertyType;
    /**
     * The type of the column.
     */
    private _type;
    /**
     * Maximum length in the database.
     */
    private _length;
    /**
     * Indicates if this column is primary key.
     */
    private _isPrimary;
    /**
     * Indicates if this column is auto increment.
     */
    private _isAutoIncrement;
    /**
     * Indicates if value should be unique or not.
     */
    private _isUnique;
    /**
     * Indicates if can contain nulls or not.
     */
    private _isNullable;
    /**
     * Indicates if column will contain a created date or not.
     */
    private _isCreateDate;
    /**
     * Indicates if column will contain an updated date or not.
     */
    private _isUpdateDate;
    /**
     * Indicates if column will contain an updated date or not.
     */
    private _isVirtual;
    /**
     * Extra sql definition for the given column.
     */
    private _columnDefinition;
    /**
     * Column comment.
     */
    private _comment;
    /**
     * Old column name. Used to correctly alter tables when column name is changed.
     */
    private _oldColumnName;
    /**
     * The precision for a decimal (exact numeric) column (applies only for decimal column), which is the maximum
     * number of digits that are stored for the values.
     */
    private _precision;
    /**
     * The scale for a decimal (exact numeric) column (applies only for decimal column), which represents the number
     * of digits to the right of the decimal point and must not be greater than precision.
     */
    private _scale;
    /**
     * Column collation. Note that not all databases support it.
     */
    private _collation;
    constructor(args: ColumnMetadataArgs);
    /**
     * Column name in the database.
     */
    name: string;
    /**
     * The real reflected property type.
     */
    propertyType: string;
    /**
     * Type of the column.
     */
    type: ColumnType;
    /**
     * Column type's length. For example type = "string" and length = 100 means that ORM will create a column with
     * type varchar(100).
     */
    length: string;
    /**
     * Indicates if this column is a primary key.
     */
    isPrimary: boolean;
    /**
     * Indicates if this column's value is auto incremented.
     */
    isAutoIncrement: boolean;
    /**
     * Indicates if this column has unique key.
     */
    isUnique: boolean;
    /**
     * Indicates if this column can have a NULL value.
     */
    isNullable: boolean;
    /**
     * Indicates if this column is special and contains object create date.
     */
    isCreateDate: boolean;
    /**
     * Indicates if this column is special and contains object last update date.
     */
    isUpdateDate: boolean;
    /**
     * Indicates if this column is virtual. Virtual column mean that it does not really exist in class. Virtual columns
     * are used for many-to-many tables.
     */
    isVirtual: boolean;
    /**
     * Extra column definition value.
     */
    columnDefinition: string;
    /**
     * Extra column's comment.
     */
    comment: string;
    /**
     * Column name used previously for this column. Used to make safe schema updates. Experimental and most probably
     * will be removed in the future. Avoid using it.
     */
    oldColumnName: string;
    /**
     * The precision for a decimal (exact numeric) column (applies only for decimal column), which is the maximum
     * number of digits that are stored for the values.
     */
    precision: number;
    /**
     * The scale for a decimal (exact numeric) column (applies only for decimal column), which represents the number
     * of digits to the right of the decimal point and must not be greater than precision.
     */
    scale: number;
    /**
     * Column collation. Note that not all databases support it.
     */
    collation: string;
}
