"use strict";
var RelationTypes_1 = require("../types/RelationTypes");
/**
 * Contains all entity metadata.
 */
var EntityMetadata = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityMetadata(table, columns, relations, indices, compoundIndices, foreignKeys) {
        this._table = table;
        this._columns = columns;
        this._relations = relations;
        this._indices = indices;
        this._compoundIndices = compoundIndices;
        this._foreignKeys = foreignKeys;
    }
    Object.defineProperty(EntityMetadata.prototype, "name", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        get: function () {
            return this._table.target.name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "target", {
        get: function () {
            return this._table.target;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "table", {
        get: function () {
            return this._table;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "columns", {
        get: function () {
            return this._columns;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "relations", {
        get: function () {
            return this._relations;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "foreignKeys", {
        get: function () {
            return this._foreignKeys;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToOneRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerOneToOneRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToManyRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToOneRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToManyRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerManyToManyRelations", {
        get: function () {
            return this._relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "indices", {
        get: function () {
            return this._indices;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "compoundIndices", {
        get: function () {
            return this._compoundIndices;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "primaryColumn", {
        get: function () {
            return this._columns.find(function (column) { return column.isPrimary; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "createDateColumn", {
        get: function () {
            return this._columns.find(function (column) { return column.isCreateDate; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "updateDateColumn", {
        get: function () {
            return this._columns.find(function (column) { return column.isUpdateDate; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasPrimaryKey", {
        get: function () {
            return !!this.primaryColumn;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new entity.
     */
    EntityMetadata.prototype.create = function () {
        return new this.table.target();
    };
    EntityMetadata.prototype.createPropertiesMap = function () {
        var entity = {};
        this._columns.forEach(function (column) { return entity[column.name] = column.name; });
        this._relations.forEach(function (relation) { return entity[relation.name] = relation.name; });
        return entity;
    };
    EntityMetadata.prototype.getEntityId = function (entity) {
        return entity[this.primaryColumn.propertyName];
    };
    EntityMetadata.prototype.hasColumnWithPropertyName = function (propertyName) {
        return !!this.findColumnWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasColumnWithDbName = function (name) {
        return !!this.findColumnWithDbName(name);
    };
    EntityMetadata.prototype.findColumnWithPropertyName = function (propertyName) {
        return this._columns.find(function (column) { return column.propertyName === propertyName; });
    };
    EntityMetadata.prototype.findColumnWithDbName = function (name) {
        return this._columns.find(function (column) { return column.name === name; });
    };
    EntityMetadata.prototype.hasRelationWithPropertyName = function (propertyName) {
        return !!this.findRelationWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithDbName = function (dbName) {
        return !!this.findRelationWithDbName(dbName);
    };
    EntityMetadata.prototype.findRelationWithPropertyName = function (propertyName) {
        return this._relations.find(function (relation) { return relation.propertyName === propertyName; });
    };
    EntityMetadata.prototype.findRelationWithDbName = function (propertyName) {
        return this._relations.find(function (relation) { return relation.name === propertyName; });
    };
    EntityMetadata.prototype.findRelationWithOneWithPropertyName = function (propertyName) {
        return this._relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithOneWithDbName = function (name) {
        return this._relations.find(function (relation) { return relation.name === name && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithManyWithPropertyName = function (propertyName) {
        return this._relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.findRelationWithManyWithDbName = function (name) {
        return this._relations.find(function (relation) { return relation.name === name && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.hasRelationWithOneWithPropertyName = function (propertyName) {
        return !!this.findRelationWithOneWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithManyWithPropertyName = function (propertyName) {
        return !!this.findRelationWithManyWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithOneWithName = function (name) {
        return !!this.findRelationWithOneWithDbName(name);
    };
    EntityMetadata.prototype.hasRelationWithManyWithName = function (name) {
        return !!this.findRelationWithManyWithDbName(name);
    };
    return EntityMetadata;
}());
exports.EntityMetadata = EntityMetadata;
//# sourceMappingURL=EntityMetadata.js.map