import { PropertyMetadata } from "./PropertyMetadata";
import { RelationType } from "../types/RelationTypes";
import { RelationOptions } from "../options/RelationOptions";
import { NamingStrategy } from "../../naming-strategy/NamingStrategy";
import { EntityMetadata } from "./EntityMetadata";
import { OnDeleteType } from "./ForeignKeyMetadata";
/**
 * Function that returns a type of the field. Returned value must be a class used on the relation.
 */
export declare type RelationTypeInFunction = ((type?: any) => Function);
/**
 * Contains the name of the property of the object, or the function that returns this name.
 */
export declare type PropertyTypeInFunction<T> = string | ((t: T) => string | any);
/**
 * Relation metadata constructor arguments.
 */
export interface RelationMetadataArgs {
    /**
     * Class to which this relation is applied.
     */
    target: Function;
    /**
     * Class's property name to which this relation is applied.
     */
    propertyName: string;
    /**
     * Type of relation. Can be one of the value of the RelationTypes class.
     */
    relationType: RelationType;
    /**
     * Type of the relation. This type is in function because of language specifics and problems with recursive
     * referenced classes.
     */
    type: RelationTypeInFunction;
    /**
     * Inverse side of the relation.
     */
    inverseSideProperty: PropertyTypeInFunction<any>;
    /**
     * Indicates if this relation is owner side of the relation between entities.
     */
    isOwning: boolean;
    /**
     * Additional relation options.
     */
    options: RelationOptions;
}
/**
 * This metadata interface contains all information about some document's relation.
 */
export declare class RelationMetadata extends PropertyMetadata {
    /**
     * Naming strategy used to generate and normalize column name.
     */
    namingStrategy: NamingStrategy;
    /**
     * Column name for this relation.
     */
    private _name;
    /**
     * Relation type.
     */
    private _relationType;
    /**
     * The type of the field.
     */
    private _type;
    /**
     * Inverse side of the relation.
     */
    private _inverseSideProperty;
    /**
     * Indicates if this side is an owner of this relation.
     */
    private _isOwning;
    /**
     * If set to true then it means that related object can be allowed to be inserted to the db.
     */
    private _isCascadeInsert;
    /**
     * If set to true then it means that related object can be allowed to be updated in the db.
     */
    private _isCascadeUpdate;
    /**
     * If set to true then it means that related object can be allowed to be remove from the db.
     */
    private _isCascadeRemove;
    /**
     * Indicates if relation column value can be nullable or not.
     */
    private _isNullable;
    /**
     * Old column name.
     */
    private _oldColumnName;
    /**
     * Related entity metadata.
     */
    private _relatedEntityMetadata;
    /**
     * Junction entity metadata.
     */
    private _junctionEntityMetadata;
    /**
     * What to do with a relation on deletion of the row containing a foreign key.
     */
    private _onDelete;
    constructor(args: RelationMetadataArgs);
    name: string;
    relatedEntityMetadata: EntityMetadata;
    junctionEntityMetadata: EntityMetadata;
    relationType: RelationType;
    type: Function;
    inverseSideProperty: string;
    inverseRelation: RelationMetadata;
    isOwning: boolean;
    isCascadeInsert: boolean;
    isCascadeUpdate: boolean;
    isCascadeRemove: boolean;
    isOneToOne: boolean;
    isOneToMany: boolean;
    isManyToOne: boolean;
    isManyToMany: boolean;
    isNullable: boolean;
    oldColumnName: string;
    onDelete: OnDeleteType;
    private computeInverseSide(inverseSide);
}
