"use strict";
var PersistOperation_1 = require("./operation/PersistOperation");
var InsertOperation_1 = require("./operation/InsertOperation");
var UpdateByRelationOperation_1 = require("./operation/UpdateByRelationOperation");
var UpdateOperation_1 = require("./operation/UpdateOperation");
var CascadesNotAllowedError_1 = require("./error/CascadesNotAllowedError");
var RemoveOperation_1 = require("./operation/RemoveOperation");
/**
  * 1. collect all exist objects from the db entity
  * 2. collect all objects from the new entity
  * 3. first need to go throw all relations of the new entity and:
  *      3.1. find all objects that are new (e.g. cascade="insert") by comparing ids from the exist objects
  *      3.2. check if relation has rights to do cascade operation and throw exception if it cannot
  *      3.3. save new objects for insert operation
  * 4. second need to go throw all relations of the db entity and:
  *      4.1. find all objects that are removed (e.g. cascade="remove") by comparing data with collected objects of the new entity
  *      4.2. check if relation has rights to do cascade operation and throw exception if it cannot
  *      4.3. save new objects for remove operation
  * 5. third need to go throw collection of all new entities
  *      5.1. compare with entities from the collection of db entities, find difference and generate a change set
  *      5.2. check if relation has rights to do cascade operation and throw exception if it cannot
  *      5.3.
  * 6. go throw all relations and find junction
  *      6.1.
 *
  * if relation has "all" then all of above:
  * if relation has "insert" it can insert a new entity
  * if relation has "update" it can only update related entity
  * if relation has "remove" it can only remove related entity
 */
var EntityPersistOperationBuilder = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityPersistOperationBuilder(connection) {
        this.connection = connection;
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this.strictCascadesMode = false;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Finds columns and relations from entity2 which does not exist or does not match in entity1.
     */
    EntityPersistOperationBuilder.prototype.buildFullPersistment = function (metadata, dbEntity, persistedEntity, dbEntities, allPersistedEntities) {
        // const dbEntities = this.extractObjectsById(dbEntity, metadata);
        // const allPersistedEntities = this.extractObjectsById(persistedEntity, metadata);
        var persistOperation = new PersistOperation_1.PersistOperation();
        persistOperation.dbEntity = dbEntity;
        persistOperation.persistedEntity = persistedEntity;
        persistOperation.allDbEntities = dbEntities;
        persistOperation.allPersistedEntities = allPersistedEntities;
        persistOperation.inserts = this.findCascadeInsertedEntities(persistedEntity, dbEntities, null);
        persistOperation.updates = this.findCascadeUpdateEntities(metadata, dbEntity, persistedEntity, null);
        persistOperation.junctionInserts = this.findJunctionInsertOperations(metadata, persistedEntity, dbEntities);
        persistOperation.updatesByRelations = this.updateRelations(persistOperation.inserts, persistedEntity);
        persistOperation.removes = this.findCascadeRemovedEntities(metadata, dbEntity, allPersistedEntities, null, null, null);
        persistOperation.junctionRemoves = this.findJunctionRemoveOperations(metadata, dbEntity, allPersistedEntities);
        return persistOperation;
    };
    /**
     * Finds columns and relations from entity2 which does not exist or does not match in entity1.
     */
    EntityPersistOperationBuilder.prototype.buildOnlyRemovement = function (metadata, dbEntity, persistedEntity, dbEntities, allPersistedEntities) {
        // const dbEntities = this.extractObjectsById(dbEntity, metadata);
        // const allEntities = this.extractObjectsById(newEntity, metadata);
        var persistOperation = new PersistOperation_1.PersistOperation();
        persistOperation.dbEntity = dbEntity;
        persistOperation.persistedEntity = persistedEntity;
        persistOperation.allDbEntities = dbEntities;
        persistOperation.allPersistedEntities = allPersistedEntities;
        persistOperation.removes = this.findCascadeRemovedEntities(metadata, dbEntity, allPersistedEntities, null, null, null);
        persistOperation.junctionRemoves = this.findJunctionRemoveOperations(metadata, dbEntity, allPersistedEntities);
        return persistOperation;
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    EntityPersistOperationBuilder.prototype.findCascadeInsertedEntities = function (newEntity, dbEntities, fromRelation, operations) {
        var _this = this;
        if (operations === void 0) { operations = []; }
        var metadata = this.connection.getEntityMetadata(newEntity.constructor);
        var isObjectNew = !this.findEntityWithId(dbEntities, metadata.target, newEntity[metadata.primaryColumn.name]);
        // if object is new and should be inserted, we check if cascades are allowed before add it to operations list
        if (isObjectNew && !this.checkCascadesAllowed("insert", metadata, fromRelation)) {
            return operations; // looks like object is new here, but cascades are not allowed - then we should stop iteration
        }
        else if (isObjectNew && !operations.find(function (o) { return o.entity === newEntity; })) {
            operations.push(new InsertOperation_1.InsertOperation(newEntity));
        }
        metadata.relations
            .filter(function (relation) { return !!newEntity[relation.propertyName]; })
            .forEach(function (relation) {
            var value = newEntity[relation.propertyName];
            if (value instanceof Array) {
                value.forEach(function (subEntity) {
                    _this.findCascadeInsertedEntities(subEntity, dbEntities, relation, operations);
                });
            }
            else {
                _this.findCascadeInsertedEntities(value, dbEntities, relation, operations);
            }
        });
        return operations;
    };
    EntityPersistOperationBuilder.prototype.findCascadeUpdateEntities = function (metadata, dbEntity, newEntity, fromRelation) {
        var _this = this;
        var operations = [];
        if (!dbEntity)
            return operations;
        var diff = this.diffColumns(metadata, newEntity, dbEntity);
        if (diff.length && !this.checkCascadesAllowed("update", metadata, fromRelation)) {
            return operations;
        }
        else if (diff.length) {
            var entityId = newEntity[metadata.primaryColumn.name];
            operations.push(new UpdateOperation_1.UpdateOperation(newEntity, entityId, diff));
        }
        metadata.relations
            .filter(function (relation) { return newEntity[relation.propertyName] && dbEntity[relation.propertyName]; })
            .forEach(function (relation) {
            var relMetadata = relation.relatedEntityMetadata;
            var relationIdColumnName = relMetadata.primaryColumn.name;
            var value = newEntity[relation.propertyName];
            var dbValue = dbEntity[relation.propertyName];
            if (value instanceof Array) {
                value.forEach(function (subEntity) {
                    var subDbEntity = dbValue.find(function (subDbEntity) {
                        return subDbEntity[relationIdColumnName] === subEntity[relationIdColumnName];
                    });
                    var relationOperations = _this.findCascadeUpdateEntities(relMetadata, subDbEntity, subEntity, relation);
                    operations = operations.concat(relationOperations);
                });
            }
            else {
                var relationOperations = _this.findCascadeUpdateEntities(relMetadata, dbValue, newEntity[relation.propertyName], relation);
                operations = operations.concat(relationOperations);
            }
        });
        return operations;
    };
    EntityPersistOperationBuilder.prototype.findCascadeRemovedEntities = function (metadata, dbEntity, allPersistedEntities, fromRelation, fromMetadata, fromEntityId, parentAlreadyRemoved) {
        var _this = this;
        if (parentAlreadyRemoved === void 0) { parentAlreadyRemoved = false; }
        var operations = [];
        if (!dbEntity)
            return operations;
        var entityId = dbEntity[metadata.primaryColumn.name];
        var isObjectRemoved = parentAlreadyRemoved || !this.findEntityWithId(allPersistedEntities, metadata.target, entityId);
        // if object is removed and should be removed, we check if cascades are allowed before add it to operations list
        if (isObjectRemoved && !this.checkCascadesAllowed("remove", metadata, fromRelation)) {
            return operations; // looks like object is removed here, but cascades are not allowed - then we should stop iteration
        }
        else if (isObjectRemoved) {
            operations.push(new RemoveOperation_1.RemoveOperation(dbEntity, entityId, fromMetadata, fromRelation, fromEntityId));
        }
        metadata.relations
            .filter(function (relation) { return !!dbEntity[relation.propertyName]; })
            .forEach(function (relation) {
            var dbValue = dbEntity[relation.propertyName];
            var relMetadata = relation.relatedEntityMetadata;
            if (dbValue instanceof Array) {
                dbValue.forEach(function (subDbEntity) {
                    var relationOperations = _this.findCascadeRemovedEntities(relMetadata, subDbEntity, allPersistedEntities, relation, metadata, dbEntity[metadata.primaryColumn.name], isObjectRemoved);
                    operations = operations.concat(relationOperations);
                });
            }
            else {
                var relationOperations = _this.findCascadeRemovedEntities(relMetadata, dbValue, allPersistedEntities, relation, metadata, dbEntity[metadata.primaryColumn.name], isObjectRemoved);
                operations = operations.concat(relationOperations);
            }
        }, []);
        return operations;
    };
    /**
     * To update relation, you need:
     *   update table where this relation (owner side)
     *   set its relation property to inserted id
     *   where
     *
     */
    EntityPersistOperationBuilder.prototype.updateRelations = function (insertOperations, newEntity) {
        var _this = this;
        return insertOperations.reduce(function (operations, insertOperation) {
            return operations.concat(_this.findRelationsWithEntityInside(insertOperation, newEntity));
        }, []);
    };
    EntityPersistOperationBuilder.prototype.findRelationsWithEntityInside = function (insertOperation, entityToSearchIn) {
        var _this = this;
        var metadata = this.connection.getEntityMetadata(entityToSearchIn.constructor);
        return metadata.relations.reduce(function (operations, relation) {
            var value = entityToSearchIn[relation.propertyName];
            if (value instanceof Array) {
                value.forEach(function (sub) {
                    if (!relation.isManyToMany && sub === insertOperation.entity)
                        operations.push(new UpdateByRelationOperation_1.UpdateByRelationOperation(entityToSearchIn, insertOperation, relation));
                    var subOperations = _this.findRelationsWithEntityInside(insertOperation, sub);
                    operations.concat(subOperations);
                });
            }
            else if (value) {
                if (value === insertOperation.entity) {
                    operations.push(new UpdateByRelationOperation_1.UpdateByRelationOperation(entityToSearchIn, insertOperation, relation));
                }
                var subOperations = _this.findRelationsWithEntityInside(insertOperation, value);
                operations.concat(subOperations);
            }
            return operations;
        }, []);
    };
    EntityPersistOperationBuilder.prototype.findJunctionInsertOperations = function (metadata, newEntity, dbEntities) {
        var _this = this;
        var dbEntity = dbEntities.find(function (dbEntity) {
            return dbEntity.id === newEntity[metadata.primaryColumn.name] && dbEntity.entity.constructor === metadata.target;
        });
        return metadata.relations
            .filter(function (relation) { return relation.isManyToMany; })
            .filter(function (relation) { return newEntity[relation.propertyName] instanceof Array; })
            .reduce(function (operations, relation) {
            var relationMetadata = relation.relatedEntityMetadata;
            var relationIdProperty = relationMetadata.primaryColumn.name;
            newEntity[relation.propertyName].map(function (subEntity) {
                var has = !dbEntity ||
                    !dbEntity.entity[relation.propertyName] ||
                    !dbEntity.entity[relation.propertyName].find(function (e) { return e[relationIdProperty] === subEntity[relationIdProperty]; });
                if (has) {
                    operations.push({
                        metadata: relation.junctionEntityMetadata,
                        entity1: newEntity,
                        entity2: subEntity
                    });
                }
                var subOperations = _this.findJunctionInsertOperations(relationMetadata, subEntity, dbEntities);
                operations = operations.concat(subOperations);
            });
            return operations;
        }, []);
    };
    EntityPersistOperationBuilder.prototype.findJunctionRemoveOperations = function (metadata, dbEntity, newEntities) {
        var _this = this;
        if (!dbEntity)
            return [];
        var newEntity = newEntities.find(function (newEntity) {
            return newEntity.id === dbEntity[metadata.primaryColumn.name] && newEntity.entity.constructor === metadata.target;
        });
        return metadata.relations
            .filter(function (relation) { return relation.isManyToMany; })
            .filter(function (relation) { return dbEntity[relation.propertyName] instanceof Array; })
            .reduce(function (operations, relation) {
            var relationMetadata = relation.relatedEntityMetadata;
            var relationIdProperty = relationMetadata.primaryColumn.name;
            dbEntity[relation.propertyName].map(function (subEntity) {
                var has = !newEntity ||
                    !newEntity.entity[relation.propertyName] ||
                    !newEntity.entity[relation.propertyName].find(function (e) { return e[relationIdProperty] === subEntity[relationIdProperty]; });
                if (has) {
                    operations.push({
                        metadata: relation.junctionEntityMetadata,
                        entity1: dbEntity,
                        entity2: subEntity
                    });
                }
                var subOperations = _this.findJunctionRemoveOperations(relationMetadata, subEntity, newEntities);
                operations = operations.concat(subOperations);
            });
            return operations;
        }, []);
    };
    EntityPersistOperationBuilder.prototype.diffColumns = function (metadata, newEntity, dbEntity) {
        return metadata.columns
            .filter(function (column) { return !column.isVirtual && !column.isUpdateDate && !column.isCreateDate; })
            .filter(function (column) { return newEntity[column.propertyName] !== dbEntity[column.propertyName]; });
    };
    EntityPersistOperationBuilder.prototype.findEntityWithId = function (entityWithIds, entityClass, id) {
        return entityWithIds.find(function (entityWithId) { return entityWithId.id === id && entityWithId.entity.constructor === entityClass; });
    };
    EntityPersistOperationBuilder.prototype.checkCascadesAllowed = function (type, metadata, relation) {
        if (!relation)
            return true;
        var isAllowed = false;
        switch (type) {
            case "insert":
                isAllowed = relation.isCascadeInsert;
                break;
            case "update":
                isAllowed = relation.isCascadeUpdate;
                break;
            case "remove":
                isAllowed = relation.isCascadeRemove;
                break;
        }
        if (isAllowed === false && this.strictCascadesMode)
            throw new CascadesNotAllowedError_1.CascadesNotAllowedError(type, metadata, relation);
        return isAllowed;
    };
    return EntityPersistOperationBuilder;
}());
exports.EntityPersistOperationBuilder = EntityPersistOperationBuilder;
//# sourceMappingURL=EntityPersistOperationsBuilder.js.map