"use strict";
var Broadcaster_1 = require("../subscriber/Broadcaster");
/**
 * Executes PersistOperation in the given connection.
 */
var PersistOperationExecutor = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function PersistOperationExecutor(connection) {
        this.connection = connection;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Executes given persist operation.
     */
    PersistOperationExecutor.prototype.executePersistOperation = function (persistOperation) {
        var _this = this;
        var broadcaster = new Broadcaster_1.Broadcaster(this.connection);
        return Promise.resolve()
            .then(function () { return _this.broadcastBeforeEvents(broadcaster, persistOperation); })
            .then(function () { return _this.connection.driver.beginTransaction(); })
            .then(function () { return _this.executeInsertOperations(persistOperation); })
            .then(function () { return _this.executeInsertJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeRemoveJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateRelationsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateOperations(persistOperation); })
            .then(function () { return _this.executeRemoveRelationOperations(persistOperation); })
            .then(function () { return _this.executeRemoveOperations(persistOperation); })
            .then(function () { return _this.connection.driver.endTransaction(); })
            .then(function () { return _this.updateIdsOfInsertedEntities(persistOperation); })
            .then(function () { return _this.updateIdsOfRemovedEntities(persistOperation); })
            .then(function () { return _this.broadcastAfterEvents(broadcaster, persistOperation); });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Broadcast all before persistment events - beforeInsert, beforeUpdate and beforeRemove events.
     */
    PersistOperationExecutor.prototype.broadcastBeforeEvents = function (broadcaster, persistOperation) {
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            return broadcaster.broadcastBeforeInsertEvent(persistedEntityWithId.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            return broadcaster.broadcastBeforeUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return broadcaster.broadcastBeforeRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Broadcast all after persistment events - afterInsert, afterUpdate and afterRemove events.
     */
    PersistOperationExecutor.prototype.broadcastAfterEvents = function (broadcaster, persistOperation) {
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntity = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            return broadcaster.broadcastAfterInsertEvent(persistedEntity.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            return broadcaster.broadcastAfterUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return broadcaster.broadcastAfterRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Executes insert operations.
     */
    PersistOperationExecutor.prototype.executeInsertOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.inserts.map(function (operation) {
            return _this.insert(operation.entity).then(function (result) {
                operation.entityId = result.insertId;
            });
        }));
    };
    /**
     * Executes insert junction operations.
     */
    PersistOperationExecutor.prototype.executeInsertJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionInserts.map(function (junctionOperation) {
            return _this.insertJunctions(junctionOperation, persistOperation.inserts);
        }));
    };
    /**
     * Executes remove junction operations.
     */
    PersistOperationExecutor.prototype.executeRemoveJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionRemoves.map(function (junctionOperation) {
            return _this.removeJunctions(junctionOperation);
        }));
    };
    /**
     * Executes update relations operations.
     */
    PersistOperationExecutor.prototype.executeUpdateRelationsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updatesByRelations.map(function (updateByRelation) {
            _this.updateByRelation(updateByRelation, persistOperation.inserts);
        }));
    };
    /**
     * Executes update operations.
     */
    PersistOperationExecutor.prototype.executeUpdateOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updates.map(function (updateOperation) {
            return _this.update(updateOperation);
        }));
    };
    /**
     * Executes remove relations operations.
     */
    PersistOperationExecutor.prototype.executeRemoveRelationOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes
            .filter(function (operation) { return operation.relation && !operation.relation.isManyToMany && !operation.relation.isOneToMany; })
            .map(function (operation) {
            return _this.updateDeletedRelations(operation);
        }));
    };
    /**
     * Executes remove operations.
     */
    PersistOperationExecutor.prototype.executeRemoveOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes.map(function (operation) {
            return _this.delete(operation.entity);
        }));
    };
    /**
     * Updates all ids of the inserted entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfInsertedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.inserts.forEach(function (insertOperation) {
            var metadata = _this.connection.getEntityMetadata(insertOperation.entity.constructor);
            insertOperation.entity[metadata.primaryColumn.name] = insertOperation.entityId;
        });
    };
    /**
     * Removes all ids of the removed entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfRemovedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.removes.forEach(function (removeOperation) {
            var metadata = _this.connection.getEntityMetadata(removeOperation.entity.constructor);
            var removedEntity = persistOperation.allPersistedEntities.find(function (allNewEntity) {
                return allNewEntity.entity.constructor === removeOperation.entity.constructor && allNewEntity.id === removeOperation.entity[metadata.primaryColumn.name];
            });
            if (removedEntity)
                removedEntity.entity[metadata.primaryColumn.propertyName] = undefined;
        });
    };
    PersistOperationExecutor.prototype.updateByRelation = function (operation, insertOperations) {
        var tableName, relationName, relationId, idColumn, id;
        var idInInserts = insertOperations.find(function (o) { return o.entity === operation.targetEntity; }).entityId;
        if (operation.updatedRelation.isOneToMany) {
            var metadata = this.connection.getEntityMetadata(operation.insertOperation.entity.constructor);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.inverseRelation.name;
            relationId = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            idColumn = metadata.primaryColumn.name;
            id = operation.insertOperation.entityId;
        }
        else {
            var metadata = this.connection.getEntityMetadata(operation.targetEntity.constructor);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.name;
            relationId = operation.insertOperation.entityId;
            idColumn = metadata.primaryColumn.name;
            id = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
        }
        return this.connection.driver.update(tableName, (_a = {}, _a[relationName] = relationId, _a), (_b = {}, _b[idColumn] = id, _b));
        var _a, _b;
    };
    PersistOperationExecutor.prototype.update = function (updateOperation) {
        var _this = this;
        var entity = updateOperation.entity;
        var metadata = this.connection.getEntityMetadata(entity.constructor);
        var values = updateOperation.columns.reduce(function (object, column) {
            var value = _this.connection.driver.preparePersistentValue(entity[column.propertyName], column);
            object[column.name] = value;
            return object;
        }, {});
        if (metadata.updateDateColumn)
            values[metadata.updateDateColumn.name] = this.connection.driver.preparePersistentValue(new Date(), metadata.updateDateColumn);
        return this.connection.driver.update(metadata.table.name, values, (_a = {}, _a[metadata.primaryColumn.name] = metadata.getEntityId(entity), _a));
        var _a;
    };
    PersistOperationExecutor.prototype.updateDeletedRelations = function (removeOperation) {
        return this.connection.driver.update(removeOperation.fromMetadata.table.name, (_a = {}, _a[removeOperation.relation.name] = null, _a), (_b = {}, _b[removeOperation.fromMetadata.primaryColumn.name] = removeOperation.fromEntityId, _b));
        var _a, _b;
    };
    PersistOperationExecutor.prototype.delete = function (entity) {
        var metadata = this.connection.getEntityMetadata(entity.constructor);
        return this.connection.driver.delete(metadata.table.name, (_a = {}, _a[metadata.primaryColumn.name] = entity[metadata.primaryColumn.propertyName], _a));
        var _a;
    };
    PersistOperationExecutor.prototype.insert = function (entity) {
        var _this = this;
        var metadata = this.connection.getEntityMetadata(entity.constructor);
        var columns = metadata.columns
            .filter(function (column) { return !column.isVirtual; })
            .filter(function (column) { return entity.hasOwnProperty(column.propertyName); })
            .map(function (column) { return column.name; });
        var values = metadata.columns
            .filter(function (column) { return !column.isVirtual; })
            .filter(function (column) { return entity.hasOwnProperty(column.propertyName); })
            .map(function (column) { return _this.connection.driver.preparePersistentValue(entity[column.propertyName], column); })
            .map(function (value) {
            if (value === null || value === undefined) {
                return "NULL";
            }
            else {
                return _this.connection.driver.escape(value);
            }
        });
        var relationColumns = metadata.relations
            .filter(function (relation) { return relation.isOwning && !!relation.relatedEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .filter(function (relation) { return entity[relation.propertyName][relation.relatedEntityMetadata.primaryColumn.name]; })
            .map(function (relation) { return relation.name; });
        var relationValues = metadata.relations
            .filter(function (relation) { return relation.isOwning && !!relation.relatedEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .filter(function (relation) { return entity[relation.propertyName].hasOwnProperty(relation.relatedEntityMetadata.primaryColumn.name); })
            .map(function (relation) { return _this.connection.driver.escape(entity[relation.propertyName][relation.relatedEntityMetadata.primaryColumn.name]); });
        var allColumns = columns.concat(relationColumns);
        var allValues = values.concat(relationValues);
        if (metadata.createDateColumn) {
            allColumns.push(metadata.createDateColumn.name);
            allValues.push(this.connection.driver.escape(this.connection.driver.preparePersistentValue(new Date(), metadata.createDateColumn)));
        }
        if (metadata.updateDateColumn) {
            allColumns.push(metadata.updateDateColumn.name);
            allValues.push(this.connection.driver.escape(this.connection.driver.preparePersistentValue(new Date(), metadata.updateDateColumn)));
        }
        return this.connection.driver.insert(metadata.table.name, this.zipObject(allColumns, allValues));
    };
    PersistOperationExecutor.prototype.insertJunctions = function (junctionOperation, insertOperations) {
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.connection.getEntityMetadata(junctionOperation.entity1.constructor);
        var metadata2 = this.connection.getEntityMetadata(junctionOperation.entity2.constructor);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var id1 = junctionOperation.entity1[metadata1.primaryColumn.name] || insertOperations.find(function (o) { return o.entity === junctionOperation.entity1; }).entityId;
        var id2 = junctionOperation.entity2[metadata2.primaryColumn.name] || insertOperations.find(function (o) { return o.entity === junctionOperation.entity2; }).entityId;
        var values = [id1, id2]; // todo: order may differ, find solution (column.table to compare with entity metadata table?)
        return this.connection.driver.insert(junctionMetadata.table.name, this.zipObject(columns, values));
    };
    PersistOperationExecutor.prototype.removeJunctions = function (junctionOperation) {
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.connection.getEntityMetadata(junctionOperation.entity1.constructor);
        var metadata2 = this.connection.getEntityMetadata(junctionOperation.entity2.constructor);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var id1 = junctionOperation.entity1[metadata1.primaryColumn.name];
        var id2 = junctionOperation.entity2[metadata2.primaryColumn.name];
        return this.connection.driver.delete(junctionMetadata.table.name, (_a = {}, _a[columns[0]] = id1, _a[columns[1]] = id2, _a));
        var _a;
    };
    PersistOperationExecutor.prototype.zipObject = function (keys, values) {
        return keys.reduce(function (object, column, index) {
            object[column] = values[index];
            return object;
        }, {});
    };
    return PersistOperationExecutor;
}());
exports.PersistOperationExecutor = PersistOperationExecutor;
//# sourceMappingURL=PersistOperationExecutor.js.map