import { Connection } from "../connection/Connection";
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { FindOptions } from "./FindOptions";
import { Repository } from "./Repository";
import { ConstructorFunction } from "../common/ConstructorFunction";
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its method, whatever
 * entity type are you passing.
 */
export declare class EntityManager {
    private connection;
    constructor(connection: Connection);
    /**
     * Gets repository of the given entity.
     */
    getRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): Repository<Entity>;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Function): boolean;
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    createQueryBuilder<Entity>(entityClass: ConstructorFunction<Entity> | Function, alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create<Entity>(entityClass: ConstructorFunction<Entity> | Function, fromRawEntity?: Object): Entity;
    /**
     * Creates a entities from the given array of plain javascript objects.
     */
    createMany<Entity>(entityClass: ConstructorFunction<Entity> | Function, copyFromObjects: any[]): Entity[];
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize<Entity>(entityClass: ConstructorFunction<Entity> | Function, object: Object): Promise<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge<Entity>(entity1: Entity, entity2: Entity): Entity;
    /**
     * Persists (saves) a given entity in the database.
     */
    persist<Entity>(entity: Entity): Promise<Entity>;
    /**
     * Removes a given entity from the database.
     */
    remove<Entity>(entity: Entity): Promise<{}>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<Entity>;
    /**
     * Finds entity with given id.
     */
    findById<Entity>(entityClass: ConstructorFunction<Entity> | Function, id: any, options?: FindOptions): Promise<Entity>;
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query: string): Promise<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any>): Promise<any>;
}
