import { Connection } from "../connection/Connection";
import { EntityMetadata } from "../metadata-builder/metadata/EntityMetadata";
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { FindOptions } from "./FindOptions";
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 */
export declare class Repository<Entity> {
    private connection;
    private metadata;
    constructor(connection: Connection, metadata: EntityMetadata);
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Entity): boolean;
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create(fromRawEntity?: Object): Entity;
    /**
     * Creates a entities from the given array of plain javascript objects.
     */
    createMany(copyFromObjects: Object[]): Entity[];
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize(object: Object): Promise<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge(entity1: Entity, entity2: Entity): Entity;
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    persist(entity: Entity): Promise<Entity>;
    /**
     * Removes a given entity from the database.
     */
    remove(entity: Entity): Promise<Entity>;
    /**
     * Finds all entities.
     */
    find(): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object): Promise<Entity[]>;
    /**
     * Finds entities with .
     */
    find(options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(options: FindOptions): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object, options: FindOptions): Promise<{
        items: Entity[];
        count: number;
    }>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(options: FindOptions): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object, options: FindOptions): Promise<Entity>;
    /**
     * Finds entity with given id.
     */
    findById(id: any, options?: FindOptions): Promise<Entity>;
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query: string): Promise<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any>): Promise<any>;
    private createFindQueryBuilder(conditionsOrFindOptions?, options?);
    /**
     * When ORM loads dbEntity it uses joins to load all entity dependencies. However when dbEntity is newly persisted
     * to the db, but uses already exist in the db relational entities, those entities cannot be loaded, and will
     * absent in dbEntities. To fix it, we need to go throw all persistedEntities we have, find out those which have
     * ids, check if we did not load them yet and try to load them. This algorithm will make sure that all dbEntities
     * are loaded. Further it will help insert operations to work correctly.
     */
    private findNotLoadedIds(dbEntities, persistedEntities);
    /**
     * Extracts unique objects from given entity and all its downside relations.
     */
    private extractObjectsById(entity, metadata, entityWithIds?);
}
