"use strict";
/**
 * Creates indexes based on the given metadata
 */
var SchemaCreator = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function SchemaCreator(connection) {
        this.connection = connection;
        this.schemaBuilder = connection.driver.createSchemaBuilder();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates complete schemas for the given entity metadatas.
     */
    SchemaCreator.prototype.create = function () {
        var _this = this;
        var metadatas = this.connection.entityMetadatas;
        return Promise.resolve()
            .then(function (_) { return _this.dropForeignKeysForAll(metadatas); })
            .then(function (_) { return _this.createTablesForAll(metadatas); })
            .then(function (_) { return _this.updateOldColumnsForAll(metadatas); })
            .then(function (_) { return _this.dropRemovedColumnsForAll(metadatas); })
            .then(function (_) { return _this.addNewColumnsForAll(metadatas); })
            .then(function (_) { return _this.updateExistColumnsForAll(metadatas); })
            .then(function (_) { return _this.createForeignKeysForAll(metadatas); })
            .then(function (_) { return _this.updateUniqueKeysForAll(metadatas); })
            .then(function (_) { return _this.removePrimaryKeyForAll(metadatas); })
            .then(function (_) { });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    SchemaCreator.prototype.dropForeignKeysForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.dropForeignKeys(metadata.table, metadata.foreignKeys); }));
    };
    SchemaCreator.prototype.createTablesForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.createNewTable(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.updateOldColumnsForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.updateOldColumns(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.dropRemovedColumnsForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.dropRemovedColumns(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.addNewColumnsForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.addNewColumns(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.updateExistColumnsForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.updateExistColumns(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.createForeignKeysForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.createForeignKeys(metadata.table, metadata.foreignKeys); }));
    };
    SchemaCreator.prototype.updateUniqueKeysForAll = function (metadatas) {
        var _this = this;
        return Promise.all(metadatas.map(function (metadata) { return _this.updateUniqueKeys(metadata.table, metadata.columns); }));
    };
    SchemaCreator.prototype.removePrimaryKeyForAll = function (metadatas) {
        var _this = this;
        var queries = metadatas
            .filter(function (metadata) { return !metadata.primaryColumn; })
            .map(function (metadata) { return _this.removePrimaryKey(metadata.table); });
        return Promise.all(queries);
    };
    /**
     * Drops all (old) foreign keys that exist in the table, but does not exist in the metadata.
     */
    SchemaCreator.prototype.dropForeignKeys = function (table, foreignKeys) {
        var _this = this;
        return this.schemaBuilder.getTableForeignQuery(table).then(function (dbKeys) {
            var dropKeysQueries = dbKeys
                .filter(function (dbKey) { return !foreignKeys.find(function (foreignKey) { return foreignKey.name === dbKey; }); })
                .map(function (dbKey) { return _this.schemaBuilder.dropForeignKeyQuery(table.name, dbKey); });
            return Promise.all(dropKeysQueries);
        });
    };
    /**
     * Creates a new table if it does not exist.
     */
    SchemaCreator.prototype.createNewTable = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.checkIfTableExist(table.name).then(function (exist) {
            if (!exist)
                return _this.schemaBuilder.createTableQuery(table, columns);
        });
    };
    /**
     * Renames (and updates) all columns that has "oldColumnName".
     */
    SchemaCreator.prototype.updateOldColumns = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.getTableColumns(table.name).then(function (dbColumns) {
            var updates = columns
                .filter(function (column) { return !!column.oldColumnName && column.name !== column.oldColumnName; })
                .filter(function (column) { return dbColumns.indexOf(column.oldColumnName) !== -1; })
                .map(function (column) { return _this.schemaBuilder.changeColumnQuery(table.name, column.oldColumnName, column); });
            return Promise.all(updates);
        });
    };
    /**
     * Drops all columns exist (left old) in the table, but does not exist in the metadata.
     */
    SchemaCreator.prototype.dropRemovedColumns = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.getTableColumns(table.name).then(function (dbColumns) {
            var dropColumnQueries = dbColumns
                .filter(function (dbColumn) { return !columns.find(function (column) { return column.name === dbColumn; }); })
                .map(function (dbColumn) { return _this.schemaBuilder.dropColumnQuery(table.name, dbColumn); });
            return Promise.all(dropColumnQueries);
        });
    };
    /**
     * Adds columns from metadata which does not exist in the table.
     */
    SchemaCreator.prototype.addNewColumns = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.getTableColumns(table.name).then(function (dbColumns) {
            var newColumnQueries = columns
                .filter(function (column) { return dbColumns.indexOf(column.name) === -1; })
                .map(function (column) { return _this.schemaBuilder.addColumnQuery(table.name, column); });
            return Promise.all(newColumnQueries);
        });
    };
    /**
     * Update all exist columns which metadata has changed.
     */
    SchemaCreator.prototype.updateExistColumns = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.getChangedColumns(table.name, columns).then(function (changedColumns) {
            var updateQueries = changedColumns.map(function (changedColumn) {
                var column = columns.find(function (column) { return column.name === changedColumn.columnName; });
                return _this.schemaBuilder.changeColumnQuery(table.name, column.name, column, changedColumn.hasPrimaryKey);
            });
            return Promise.all(updateQueries);
        });
    };
    /**
     * Creates foreign keys which does not exist in the table yet.
     */
    SchemaCreator.prototype.createForeignKeys = function (table, foreignKeys) {
        var _this = this;
        return this.schemaBuilder.getTableForeignQuery(table).then(function (dbKeys) {
            var dropKeysQueries = foreignKeys
                .filter(function (foreignKey) { return dbKeys.indexOf(foreignKey.name) === -1; })
                .map(function (foreignKey) { return _this.schemaBuilder.addForeignKeyQuery(foreignKey); });
            return Promise.all(dropKeysQueries);
        });
    };
    /**
     * Creates unique keys which are missing in db yet, and drops unique keys which exist in the db,
     * but does not exist in the metadata anymore.
     */
    SchemaCreator.prototype.updateUniqueKeys = function (table, columns) {
        var _this = this;
        return this.schemaBuilder.getTableUniqueKeysQuery(table.name).then(function (dbKeys) {
            // first find metadata columns that should be unique and update them if they are not unique in db
            var addQueries = columns
                .filter(function (column) { return column.isUnique; })
                .filter(function (column) { return dbKeys.indexOf("uk_" + column.name) === -1; })
                .map(function (column) { return _this.schemaBuilder.addUniqueKey(table.name, column.name, "uk_" + column.name); });
            // second find columns in db that are unique, however in metadata columns they are not unique
            var dropQueries = columns
                .filter(function (column) { return !column.isUnique; })
                .filter(function (column) { return dbKeys.indexOf("uk_" + column.name) !== -1; })
                .map(function (column) { return _this.schemaBuilder.dropIndex(table.name, "uk_" + column.name); });
            return Promise.all([addQueries, dropQueries]);
        });
    };
    /**
     * Removes primary key from the table (if it was before and does not exist in the metadata anymore).
     */
    SchemaCreator.prototype.removePrimaryKey = function (table) {
        var _this = this;
        // find primary key name in db and remove it, because we don't have (anymore) primary key in the metadata
        return this.schemaBuilder.getPrimaryConstraintName(table.name).then(function (constraintName) {
            if (constraintName)
                return _this.schemaBuilder.dropIndex(table.name, constraintName);
        });
    };
    return SchemaCreator;
}());
exports.SchemaCreator = SchemaCreator;
//# sourceMappingURL=SchemaCreator.js.map